@@ -61,6 +61,10 @@
     #define FS "FAT"
 #endif
 
+/// global variable
+cmd_opt		opt;			/// cmd_opt structure defined in partclone.h
+p_dialog_mesg	m_dialog;			/// dialog format string
+
 /**
  * main functiom - for colne or restore data
  */
@@ -90,6 +94,7 @@ int main(int argc, char **argv){
     int			done = 0;
     int			s_count = 0;
     int			rescue_num = 0;
+    int			tui = 0;		/// text user interface
     char *bad_sectors_warning_msg =
     "*************************************************************************\n"
     "* WARNING: The disk has bad sector. This means physical damage on the   *\n"
@@ -99,7 +104,6 @@ int main(int argc, char **argv){
     "*************************************************************************\n";
 
     progress_bar	prog;			/// progress_bar structure defined in progress.h
-    cmd_opt		opt;			/// cmd_opt structure defined in partclone.h
     image_head		image_hdr;		/// image_head structure defined in partclone.h
 
     /**
@@ -116,6 +120,23 @@ int main(int argc, char **argv){
     //if(opt.debug)
 	open_log();
 
+    /**
+     * using Text User Interface
+     */
+    if (opt.ncurses){
+	log_mesg(1, 0, 0, debug, "Using Ncurses User Interface mode.\n");
+	tui = open_ncurses();
+	if (tui == 0){
+	    log_mesg(1, 0, 0, debug, "Open Ncurses User Interface Error.\n");
+	    opt.ncurses = 0;
+	    close_ncurses();
+	}
+    } else if (opt.dialog){
+	log_mesg(1, 0, 0, debug, "Using Dialog User Interface mode.\n");
+	m_dialog.percent = 1;
+	tui = 1;
+    }
+
     if (geteuid() != 0)
 	log_mesg(0, 1, 1, debug, "You are not logged as root. You may have \"access denied\" errors when working.\n"); 
     else
@@ -217,8 +238,8 @@ int main(int argc, char **argv){
 	    log_mesg(0, 1, 1, debug, "This is not partclone image.\n");
 
 	/// check the file system
-	if (memcmp(image_hdr.fs, FS, FS_MAGIC_SIZE) != 0)
-	    log_mesg(0, 1, 1, debug, "Partclone can't restore partition from the image because the file system is different.\n");
+	//if (strcmp(image_hdr.fs, FS) != 0)
+	//    log_mesg(0, 1, 1, debug, "%s can't restore from the image which filesystem is %s not %s\n", argv[0], image_hdr.fs, FS);
 
 	log_mesg(2, 0, 0, debug, "initial main bitmap pointer %lli\n", bitmap);
 	log_mesg(1, 0, 0, debug, "Initial image hdr - read bitmap table\n");
@@ -373,7 +394,12 @@ int main(int argc, char **argv){
 		free(buffer);
 		free(crc_buffer);
 
-		progress_update(&prog, copied, done);
+		if (opt.ncurses)
+		    Ncurses_progress_update(&prog, copied, done);
+		else if (opt.dialog)
+		    Dialog_progress_update(&prog, copied, done);
+		else
+		    progress_update(&prog, copied, done);
         	
 		copied++;					/// count copied block
 		total_write += (unsigned long long)(w_size);	/// count copied size
@@ -394,7 +420,13 @@ int main(int argc, char **argv){
 	    
 		s_count++;
 		if ((s_count >=100) || (done == 1)){
-		    progress_update(&prog, copied, done);
+		    if (opt.ncurses)
+			Ncurses_progress_update(&prog, copied, done);
+		    else if (opt.dialog)
+			Dialog_progress_update(&prog, copied, done);
+		    else
+			progress_update(&prog, copied, done);
+
 		    s_count = 0;
 		}
 		log_mesg(2, 0, 0, debug, "end\n");
@@ -482,7 +514,12 @@ int main(int argc, char **argv){
 		free(buffer);
 		free(crc_buffer);
 
-		progress_update(&prog, copied, done);
+		if (opt.ncurses)
+		    Ncurses_progress_update(&prog, copied, done);
+		else if (opt.dialog)
+		    Dialog_progress_update(&prog, copied, done);
+		else
+		    progress_update(&prog, copied, done);
 
 		copied++;					/// count copied block
 		total_write += (unsigned long long) w_size;	/// count copied size
@@ -501,7 +538,12 @@ int main(int argc, char **argv){
 		    log_mesg(0, 1, 1, debug, "seek error %lli errno=%i\n", (long long)offset, (int)errno);
 		s_count++;
 		if ((s_count >=100) || (done == 1)){
-		    progress_update(&prog, copied, done);
+		    if (opt.ncurses)
+			Ncurses_progress_update(&prog, copied, done);
+		    else if (opt.dialog)
+			Dialog_progress_update(&prog, copied, done);
+		    else
+			progress_update(&prog, copied, done);
 		    s_count = 0;
 		}
 		log_mesg(2, 0, 0, debug, "end\n");
@@ -572,7 +614,12 @@ int main(int argc, char **argv){
 
 		/// free buffer
 		free(buffer);
-		progress_update(&prog, copied, done);
+		if (opt.ncurses)
+		    Ncurses_progress_update(&prog, copied, done);
+		else if (opt.dialog)
+		    Dialog_progress_update(&prog, copied, done);
+		else
+		    progress_update(&prog, copied, done);
 		copied++;                                       /// count copied block
 		total_write += (unsigned long long)(w_size);    /// count copied size
 		log_mesg(1, 0, 0, debug, "total=%lli, ", total_write);
@@ -592,8 +639,13 @@ int main(int argc, char **argv){
 		    log_mesg(0, 1, 1, debug, "clone seek error %lli errno=%i\n", (long long)offset, (int)errno);
 		s_count++;
 		if ((s_count >=100) || (done == 1)){
-		    progress_update(&prog, copied, done);
-		     s_count = 0;
+		    if (opt.ncurses)
+			Ncurses_progress_update(&prog, copied, done);
+		    else if (opt.dialog)
+			Dialog_progress_update(&prog, copied, done);
+		    else
+			progress_update(&prog, copied, done);
+		    s_count = 0;
 		}
 
 		log_mesg(2, 0, 0, debug, "end\n");
@@ -608,6 +660,9 @@ int main(int argc, char **argv){
     close (dfr);    /// close source
     close (dfw);    /// close target
     free(bitmap);   /// free bitmp
+    if(opt.ncurses)
+	close_ncurses();
+	printf("clone successfully\n");
     if(opt.debug)
 	close_log();
     return 0;	    /// finish
