@@ -50,6 +50,14 @@
 
 
 FILE* msg = NULL;
+#ifdef HAVE_LIBNCURSESW
+    #include <ncurses.h>
+    WINDOW *log_win;
+    WINDOW *p_win;
+    WINDOW *box_win;
+    WINDOW *bar_win;
+    int log_y_line = 0;
+#endif
 
 /**
  * options - 
@@ -58,7 +66,7 @@ FILE* msg = NULL;
  */
 extern void usage(void)
 {
-    fprintf(stderr, "%s v%s (%s) partclone.sourceforge.net, partclone.nchc.org.tw\nUsage: %s [OPTIONS]\n"
+    fprintf(stderr, "%s v%s (%s) www.partclone.org, partclone.nchc.org.tw\nUsage: %s [OPTIONS]\n"
         "    Efficiently clone to a image, device or standard output.\n"
         "\n"
         "    -o,  --output FILE      Output FILE\n"
@@ -70,6 +78,10 @@ extern void usage(void)
         "    -dX, --debug=X          Set the debug level to X = [0|1|2]\n"
         "    -R,  --rescue           Continue after disk read errors\n"
         "    -C,  --no_check         Don't check device size and free space\n"
+#ifdef HAVE_LIBNCURSESW
+        "    -N,  --ncurses          Using Ncurses User Interface\n"
+#endif
+        "    -X,  --dialog           output message as Dialog Format\n"
         "    -h,  --help             Display this help\n"
     , EXECNAME, VERSION, svn_version, EXECNAME);
     exit(0);
@@ -77,7 +89,7 @@ extern void usage(void)
 
 extern void parse_options(int argc, char **argv, cmd_opt* opt)
 {
-    static const char *sopt = "-hd::cbro:O:s:RC";
+    static const char *sopt = "-hd::cbro:O:s:RCXN";
     static const struct option lopt[] = {
         { "help",		no_argument,	    NULL,   'h' },
         { "output",		required_argument,  NULL,   'o' },
@@ -89,6 +101,10 @@ extern void parse_options(int argc, char **argv, cmd_opt* opt)
         { "debug",		optional_argument,  NULL,   'd' },
         { "rescue",		no_argument,	    NULL,   'R' },
         { "check",		no_argument,	    NULL,   'C' },
+        { "dialog",		no_argument,	    NULL,   'X' },
+#ifdef HAVE_LIBNCURSESW
+        { "ncurses",		no_argument,	    NULL,   'N' },
+#endif
         { NULL,			0,		    NULL,    0  }
     };
 
@@ -136,6 +152,17 @@ extern void parse_options(int argc, char **argv, cmd_opt* opt)
 	    case 'R':
 		    opt->rescue++;
 		    break;
+	    case 'X':
+		    /// output message as dialog format, reference
+		    /// dialog --guage is text height width percent
+		    ///    A guage box displays a meter along the bottom of the box. The meter indicates the percentage. New percentages are read from standard input, one integer per line. The meter is updated to reflect each new percentage. If stdin is XXX, then the first line following is taken as an integer percentage, then subsequent lines up to another XXX are used for a new prompt. The guage exits when EOF is reached on stdin. 
+		    opt->dialog = 1;
+		    break;
+#ifdef HAVE_LIBNCURSESW
+	    case 'N':
+		    opt->ncurses = 1;
+		    break;
+#endif
 	    case 'C':
 		    opt->check = 0;
 		    break;
@@ -164,7 +191,7 @@ extern void parse_options(int argc, char **argv, cmd_opt* opt)
 	//fprintf(stderr, "You use specify output file like stdout. or --help get more info.\n");
 	opt->source = "-";
     }
-	
+
     if (opt->clone){
 
 	if ((strcmp(opt->source, "-") == 0) || (opt->source == NULL)) {
@@ -172,6 +199,14 @@ extern void parse_options(int argc, char **argv, cmd_opt* opt)
 	    //usage();
 	    exit(0);
 	}
+
+	if ((strcmp(opt->target, "-") == 0) || (opt->target == NULL)) {
+	    if (opt->ncurses){
+		fprintf(stderr, "Warning: Partclone can't save output to stdout with ncurses interface.\n");
+		opt->ncurses = 0;
+	    }
+	    
+	}
     
     }
     
@@ -188,6 +223,119 @@ extern void parse_options(int argc, char **argv, cmd_opt* opt)
 }
 
 
+/**
+ * Ncurses Text User Interface
+ * open_ncurses	    - open text window
+ * close_ncurses    - close text window
+ */
+extern int open_ncurses(){
+    int debug = 1;
+
+#ifdef HAVE_LIBNCURSESW
+    extern cmd_opt opt;
+    int terminal_x = 0;
+    int terminal_y = 0;
+    initscr();
+
+    // check terminal width and height
+    getmaxyx(stdscr, terminal_y, terminal_x);
+
+    // set window position
+    int log_line = 10;
+    int log_row = 60;
+    int log_y_pos = (terminal_y-24)/2+2;
+    int log_x_pos = (terminal_x-log_row)/2;
+    int gap = 2;
+    int p_line = 8;
+    int p_row = log_row;
+    int p_y_pos = log_y_pos+log_line+gap;
+    int p_x_pos = log_x_pos;
+
+    int size_ok = 1;
+ 
+    if(terminal_y < (log_line+gap+p_line+3))
+	size_ok = 0;
+    if(terminal_x < (log_row+2))
+	size_ok = 0;
+
+    if (size_ok == 0){
+	log_mesg(0, 0, 0, debug, "Terminal width(%i) or height(%i) too small\n", terminal_x, terminal_y);
+	return 0;
+    }
+
+    /// check color pair
+    if(!has_colors()){
+	log_mesg(0, 0, 0, debug, "Terminal color error\n");
+	return 0;
+    }
+
+    if (start_color() != OK){
+	log_mesg(0, 0, 0, debug, "Terminal can't start color mode\n");
+	return 0;
+    }
+
+    /// define color
+    init_pair(1, COLOR_WHITE, COLOR_BLUE); ///stdscr
+    init_pair(2, COLOR_RED, COLOR_WHITE); ///sub window
+    init_pair(3, COLOR_BLUE, COLOR_WHITE); ///sub window
+
+    /// write background color
+    bkgd(COLOR_PAIR(1));
+    touchwin(stdscr);
+    refresh();
+
+    /// init main box
+    attrset(COLOR_PAIR(2));
+    box_win = subwin(stdscr, (log_line+gap+p_line+2), log_row+2, log_y_pos-1, log_x_pos-1);
+    box(box_win, ACS_VLINE, ACS_HLINE);
+    wbkgd(box_win, COLOR_PAIR(2));
+    mvprintw((log_y_pos-1), ((terminal_x-9)/2), " Partclone ");
+    attroff(COLOR_PAIR(2));
+
+    attrset(COLOR_PAIR(3));
+
+    /// init log window
+    log_win = subwin(stdscr, log_line, log_row, log_y_pos, log_x_pos);
+    wbkgd(log_win, COLOR_PAIR(3));
+    wprintw(log_win, "Calculating bitmap...\nPlease wait...\n");
+
+    // init progress window
+    p_win = subwin(stdscr, p_line, p_row, p_y_pos, p_x_pos);
+    wbkgd(p_win, COLOR_PAIR(3));
+    mvwprintw(p_win, 5, 52, "  0.00%%\n");
+
+    // init progress window
+    bar_win = subwin(stdscr, 1, p_row-10, p_y_pos+5, p_x_pos);
+    wbkgd(bar_win, COLOR_PAIR(1));
+
+    scrollok(log_win, TRUE);
+
+    if( touchwin(stdscr) == ERR ){
+	return 0;
+    }
+
+    //clear();
+
+    refresh();
+
+#endif
+    return 1;
+}
+
+extern void close_ncurses(){
+#ifdef HAVE_LIBNCURSESW
+    sleep(3);
+    attroff(COLOR_PAIR(3));
+    delwin(log_win);
+    delwin(p_win);
+    delwin(bar_win);
+    delwin(box_win);
+    touchwin(stdscr);
+    endwin();
+#endif
+}
+
+
 /**
  * debug message
  * open_log	- to open file /var/log/partclone.log
@@ -208,14 +356,45 @@ extern void log_mesg(int log_level, int log_exit, int log_stderr, int debug, con
 
     va_list args;
     va_start(args, fmt);
-
+    extern cmd_opt opt;
+    extern p_dialog_mesg m_dialog;
+    char tmp_str[128];
 	
-    /// write log to stderr if log_stderr true
-    if((log_stderr) && (log_level <= debug)){
-        vfprintf(stderr, fmt, args);
+    if (opt.ncurses) {
+#ifdef HAVE_LIBNCURSESW
+	setlocale(LC_ALL, "");
+	bindtextdomain(PACKAGE, LOCALEDIR);
+	textdomain(PACKAGE);
+	if((log_stderr) && (log_level <= debug)){
+	    if(log_exit){
+		wattron(log_win, A_STANDOUT);
+	    }
+
+	    vwprintw(log_win, fmt, args);
+	    
+	    if(log_exit){
+		wattroff(log_win, A_STANDOUT);
+		sleep(3);
+	    }
+	    wrefresh(log_win);
+	    log_y_line++;
+	}
+#endif
+    } else if (opt.dialog) {
+	/// write log to stderr if log_stderr is true
+	if((log_stderr) && (log_level <= debug)){
+	    vsprintf(tmp_str, fmt, args);
+	    strncat(m_dialog.data, tmp_str, strlen(tmp_str));
+	    fprintf(stderr, "XXX\n%i\n%s\nXXX\n", m_dialog.percent, m_dialog.data);
+	}
+    } else {
+	/// write log to stderr if log_stderr is true
+	if((log_stderr) && (log_level <= debug)){
+	    vfprintf(stderr, fmt, args);
+	}
     }
 
-    /// write log to log file if debug true
+    /// write log to logfile if debug is true
     if(log_level <= debug){
 	vfprintf(msg, fmt, args);
 	//if (errno != 0)
@@ -227,8 +406,11 @@ extern void log_mesg(int log_level, int log_exit, int log_stderr, int debug, con
     fflush(msg);
 
     /// exit if lexit true
-    if (log_exit)
+    if (log_exit){
+	close_ncurses();
+	fprintf(stderr, "Partclone fail, please check /var/log/partclone.log !\n");
     	exit(1);
+    }
 }
  
 extern void close_log(){
@@ -618,7 +800,7 @@ extern void print_image_hdr_info(image_head image_hdr, cmd_opt opt){
 	bindtextdomain(PACKAGE, LOCALEDIR);
 	textdomain(PACKAGE);
 	//log_mesg(0, 0, 0, "%s v%s \n", EXEC_NAME, VERSION);
-	log_mesg(0, 0, 1, debug, _("Partclone v%s (%s) partclone.sourceforge.net, partclone.nchc.org.tw\n"), VERSION, svn_version);
+	log_mesg(0, 0, 1, debug, _("Partclone v%s (%s) www.partclone.org, partclone.nchc.org.tw\n"), VERSION, svn_version);
     if (opt.clone)
 		log_mesg(0, 0, 1, debug, _("Starting clone device (%s) to image (%s)\n"), opt.source, opt.target);	
 	else if(opt.restore)
@@ -643,11 +825,11 @@ extern void print_finish_info(cmd_opt opt){
     bindtextdomain(PACKAGE, LOCALEDIR);
     textdomain(PACKAGE);
     if (opt.clone)
-	log_mesg(0, 0, 1, debug, _("Partclone successfully clone the device (%s) to the image (%s)\n"), opt.source, opt.target);	
+	log_mesg(0, 0, 1, debug, _("Partclone successfully cloned the device (%s) to the image (%s)\n"), opt.source, opt.target);	
     else if(opt.restore)
-	log_mesg(0, 0, 1, debug, _("Partclone successfully restore the image (%s) to the device (%s)\n"), opt.source, opt.target);
+	log_mesg(0, 0, 1, debug, _("Partclone successfully restored the image (%s) to the device (%s)\n"), opt.source, opt.target);
     else if(opt.dd)
-	log_mesg(0, 0, 1, debug, _("Partclone successfully clone the device (%s) to the device (%s)\n"), opt.source, opt.target);
+	log_mesg(0, 0, 1, debug, _("Partclone successfully cloned the device (%s) to the device (%s)\n"), opt.source, opt.target);
 	    
 }
 
